%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
%  Evaluating Prior Distributions         
%==========================================================================

function [draws,prioinfo] = priodraw(prsum,rep)
% This code computes a prior density for
% the structural parameters of the DSGE models
% pshape: 0 is point mass, both para and pstdd are ignored
%         1 is BETA(mean,stdd)
%         2 is GAMMA(mean,stdd)
%         3 is NORMAL(mean,stdd)
%         4 is INVGAMMA(s^2,nu)

pmean=prsum(:,1);
pstdd=prsum(:,2);
pshape=prsum(:,3);

% local a, b, lnprior, prioinfo, nprio, i;
% lnprior = 0;
% a = 0;
% b = 0;

nprio = size(pshape,1);
prioinfo = [zeros(nprio,2),pshape];
draws=zeros(nprio,rep);

i=1;
while i<=nprio
    if prioinfo(i,3) == 1; % /* BETA Prior */
      a = (1-pmean(i,:))*pmean(i,:)^2/pstdd(i,:)^2 - pmean(i,:);
      b = a*(1/pmean(i,:) - 1);
      draws(i,:)=betarnd(a,b,1,rep);
    elseif prioinfo(i,3) == 2; % /* GAMMA PRIOR */
      b = pstdd(i,:)^2/pmean(i,:);
      a = pmean(i,:)/b;
      draws(i,:)=gamrnd(a,b,1,rep);
    elseif prioinfo(i,3) == 3; % /* GAUSSIAN PRIOR */
      a = pmean(i,:);
      b = pstdd(i,:);
      draws(i,:)=b*randn(1,rep)+repmat(a,1,rep);
    elseif prioinfo(i,3) == 4; % /* INVGAMMA PRIOR */
      a = pmean(i,:); % For inverse gamma these are already transformed
      b = pstdd(i,:);
      draws(i,:)=draw_ig1(a,b,rep);
    elseif prioinfo(i,3) == 5; % Dirichlet
      a = pmean(i,:); % For inverse gamma these are already transformed
      b = pstdd(i,:);
      draws(i,:)=drchrnd([a,b],rep);
    elseif prioinfo(i,3) == 6; % uniform
      a = pmean(i,:); % min
      b = pstdd(i,:); % max
      draws(i,:)=rand*(b-a)+a;
%       keyboard
    end
    prioinfo(i,1:2) = [a,b];
    i=i+1;
end






