%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================

function [vd,vol,check,corr,corry,covmaty,covmatS,covmaty_sh,corry_sh]=vardec5_from_MAT8_chol(MAT,method,control)


% method=1: Lyapunov equation
% method=2: Law of motion for second moments using sqrtm
% method=3: Law of motion for second moments using R and Q


[ TT,QQ,RR,HH,~,ZZ ] = from_MAT_to_T(MAT);
nsb = size(TT,3);
nsc = size(QQ,3);
nsh = size(QQ,2);
[ny,nS]  = size(ZZ);

% keyboard
vd  = zeros(ny,nsh+2,nsb*nsc);
vol = zeros(ny,nsb*nsc);
corry = zeros(ny,nsb*nsc);
corr = zeros(nS,nsb*nsc,2);
covmaty = zeros(ny,ny,nsb*nsc); 
covmatS = zeros(nS,nS,nsb*nsc);  
covmaty_sh = zeros(ny,ny,nsb*nsc,nsh); 
corry_sh = zeros(ny,nsb*nsc,nsh);
% Compute covariance matrix MS states
for ib=1:nsb
    T=TT(:,:,ib);
    R=RR(:,:,ib);

    for jc=1:nsc
        Q=QQ(:,:,jc);
        Q05=chol(Q)';
        % 1.a) All shocks
        if method==1
            RQR=R*Q*R';
            RQR=(RQR+RQR')/2;
            OME0s=real(dlyap(T,RQR));        
            check=1;
        elseif method==2
            RQR=R*Q*R';
            RQR=(RQR+RQR')/2;
            RQR05=real(RQR^0.5);
            [OME0s,check] = unc_var_ms_dsge3( T, RQR05, 1, 0, control );
        elseif method==3
            
            RQ=R*Q05;
            
            [OME0s,check] = unc_var_ms_dsge3_RQ( T, RQ, 1, 0, control );
        end
        

        if check==1
            % Compute covariance matrix for observables
            OME0str=ZZ*OME0s*ZZ';
            
            if nargout>3
                % Autocovariance
                OME1s=T*OME0s;
                corr(:,(jc-1)*nsc+ib,1)=diag(OME1s)./diag(OME0s);
                adjust=find(diag(OME0s)<10^-20);
                corr(adjust,(jc-1)*nsc+ib,1)=NaN;
                %------------------------------------------------------
                OME1str=ZZ*OME1s*ZZ';
                corry(:,(jc-1)*nsc+ib)=diag(OME1str)./diag(OME0str);
                adjust=find(diag(OME0str)<10^-20);
                corry(adjust,(jc-1)*nsc+ib,1)=NaN;
  
            end
            
            OME0all=ZZ*OME0s*ZZ'+HH;
            % 1.b) Only str shocks and no observ shock

            covmaty(:,:,(jc-1)*nsc+ib,1)=OME0str;
            covmatS(:,:,(jc-1)*nsc+ib,1)=OME0s;
            
            
            for sh = 1:nsh
                clear OME0s
                Q05r=zeros(size(Q));
                Q05r(:,sh) = Q05(:,sh);
                Qr = Q05r*Q05r';
                % Only sh shock and no observ shock

                if method==1
                    RQrR=R*Qr*R';
                    RQrR=(RQrR+RQrR')/2;
                    OME0s_r=real(dlyap(T,RQrR));
                elseif method==2
                    RQrR=R*Qr*R';
                    RQrR=(RQrR+RQrR')/2;
                    RQrR05=real(RQrR^0.5);
                    OME0s_r = unc_var_ms_dsge3( T, RQrR05, 1, 0, 0 );
                elseif method==3
                    RQr=R*Q05r;
                    OME0s_r = unc_var_ms_dsge3_RQ( T, RQr, 1, 0, 0 );
                end
                OME0str_r=ZZ*OME0s_r*ZZ';
                % Decomp of total variance
                vd(:,sh,(jc-1)*nsc+ib,1)=diag(OME0str_r)./diag(OME0all);
                % Decomp of model implied variance
                vd(:,sh,(jc-1)*nsc+ib,2)=diag(OME0str_r)./diag(OME0str);
                % Save partial covariance matrix
                covmaty_sh(:,:,(jc-1)*nsc+ib,sh)=OME0str_r;
                
                if nargout>7
                    %------------------------------------------------------
                    % Autocovariance
                    OME1s_r=T*OME0s_r;

                    %------------------------------------------------------
                    OME1str_r=ZZ*OME1s_r*ZZ';
                    corry_sh(:,(jc-1)*nsc+ib,sh)=diag(OME1str_r)./diag(OME0str_r);
                    adjust=find(diag(OME0str_r)<10^-20);
                    corry_sh(adjust,(jc-1)*nsc+ib,sh)=NaN;
                end
            end 

    %         keyboard
            % Contribution of observation errors 
            vd(:,nsh+1,(jc-1)*nsc+ib,1)=diag(HH)./diag(OME0all);
            vd(:,nsh+2,(jc-1)*nsc+ib,1)=diag(OME0str)./diag(OME0all);
            vd(:,nsh+1,(jc-1)*nsc+ib,2)=diag(HH)./diag(OME0all);
            vd(:,nsh+2,(jc-1)*nsc+ib,2)=diag(OME0str)./diag(OME0all);
            % "Model volatility" and overall volatility (including obs err)
            
            vol(:,(jc-1)*nsc+ib,1)=(diag(OME0str).^0.5);
            vol(:,(jc-1)*nsc+ib,2)=(diag(OME0all).^0.5);
            if min(isreal(vol))==0;keyboard;end
        else
            vd=[];
            vol=[];
            
        end
    end  
end
% 