%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To run this file, the vector "p_st" has to be
% augmented to include the value for the "alph" parameter Line 24: alph = p_st(8); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [GAM0,GAM1,PSI,PPI,snames] = gammas_1002(p_st,p_ss,msel)


%---------------------------------
rho        = p_st(1);
flb_adj    = p_st(2);
surp_adj    = p_st(3);

kap        = p_st(4);

psipi      = p_st(5);
psiq       = p_st(6);
rhoR       = p_st(7);

rhoq       = p_st(8);
rhos       = p_st(9);

alph1       = p_st(10);
alph2       = p_st(11);
%-----------------------------------
pdt       = p_ss(1); % Productivity growth
fub       = p_ss(2);
pi_st     = p_ss(3);

%---------------------------------------

bet = 1/(1 + rho/100);
surp = 1/(1 + surp_adj/100);
flb  = 1/(1 + flb_adj/100);
Gam = 1 + pdt/100;
% Two conditions to guarantee bubbly BGP (pag. 15)
rr_st = fub*pdt;  % Condition to guarantee non-negative newly created bubbles u>0 --> Del*Gam >1 --> rr_st < pdt 
Del = 1/(1+rr_st/100);
v = flb*bet/(Del*Gam);% Condition to guarantee existence of a BGP with positive bubbles q_b >0 --> v < bet/(Del*Gam)

Phi = (Del*Gam*v)/bet;
    
Ups = (1 - bet*surp)/(1 - Del*Gam*v*surp);
Psi = Ups*Phi* (1 + (v*surp - v*surp*Phi)/(Phi - Phi*bet*surp));
The = (1 - bet*surp)*(1-v*surp) / (bet*surp);
q_b = surp*(bet - Del*Gam*v)/((1-bet*surp)*(1-Del*Gam*v*surp));


% 
snames=cell(1,200);
% State variables
ii=1;
vy = ii;ii=ii+1;snames{ii-1}='y';
vpi = ii;ii=ii+1;snames{ii-1}='\pi';
vr = ii;ii=ii+1;snames{ii-1}='R';
vq = ii;ii=ii+1;snames{ii-1}='q';
vb = ii;ii=ii+1;snames{ii-1}='b';

vuq = ii;ii=ii+1;snames{ii-1}='uq';
vus = ii;ii=ii+1;snames{ii-1}='us';

vy1 = ii;ii=ii+1;snames{ii-1}='y1';
%%%%%%%%%%%%%%%%%% NEW
vw1 = ii;ii=ii+1;snames{ii-1}='w1';  % !!! variable omega "vw1" in aux process as additional state var
vw2 = ii;ii=ii+1;snames{ii-1}='w2';  % !!! variable omega "vw2" in aux process as additional state var
%%%%%%%%%%%%%%%%%%

% Endog variables
ie=1;
vEy = ii;ii=ii+1;ie=ie+1;snames{ii-1}='Ey';
vEpi = ii;ii=ii+1;ie=ie+1;snames{ii-1}='E\pi';
vEb = ii;ii=ii+1;ie=ie+1;snames{ii-1}='Eb';

nend = ie-1;
neq  = ii-1;
snames=snames(:,1:neq);
%------------------------------
% Exog shocks
is=1;
eR = is;is=is+1;
euq = is;is=is+1;
eus = is;is=is+1;

%%%%%%%%%%%%%%%%%% NEW
esuns1 = is;is=is+1;  % !!! Sunspot shock as additional exogenous shock (different from zeta shock in LS,2004)
esuns2 = is;is=is+1;  % !!! Sunspot shock as additional exogenous shock (different from zeta shock in LS,2004)
%%%%%%%%%%%%%%%%%%

nex  = is-1;
%------------------------------
GAM0 = zeros(neq,neq);
GAM1 = zeros(neq,neq);
PSI = zeros(neq,nex);
PPI = zeros(neq,nend);


eq=1;
% 1) IS Curve    vy+Psi*(vr-vEpi)-The*vq-Phi*vEy
GAM0(eq,vy)   = 1;
GAM0(eq,vr)   = Psi;
GAM0(eq,vq)   = -The;
GAM0(eq,vEy)  = -Phi;
GAM0(eq,vEpi) = -Psi;

eq=eq+1;
% 2) Phillips Curve   vpi-(Del*Gam*v*surp)*vEpi-kappa*vy-vus
GAM0(eq,vy)   = -kap;
GAM0(eq,vpi)  =  1;
GAM0(eq,vus)  = -1;
GAM0(eq,vEpi) =-(Del*Gam*v*surp);

eq=eq+1;
% 3) Monetary policy rule
GAM0(eq,vpi)= -(1-rhoR)*psipi;
GAM0(eq,vr) = 1;
GAM0(eq,vq) = -(1-rhoR)*psiq;  % Note:response to bubble component

GAM1(eq,vr) = rhoR;
PSI(eq,eR)  = 1;

eq=eq+1;

% 4) Bubble evolution    vq + q_b*vr - q_b*vEpi - Del*Gam*vEb
GAM0(eq,vr)   =  q_b;
GAM0(eq,vq)   =    1;
GAM0(eq,vEpi) = -q_b;
GAM0(eq,vEb)  = -Del*Gam;

eq=eq+1;

% 5) Bubble definition    vq - vb -vuq
GAM0(eq,vq) =    1;
GAM0(eq,vb) =   -1;
GAM0(eq,vuq) =  -1;

eq=eq+1;

% 6) Bubble shock
GAM0(eq,vuq)    = 1;
GAM1(eq,vuq)    = rhoq;
PSI(eq,euq)     = 1;
% PSI(4,2)     = 1;

eq=eq+1;
% 7) Technology Growth
GAM0(eq,vus)    = 1;
GAM1(eq,vus)    = rhos;
PSI(eq,eus)     = 1;
eq=eq+1;

% 8) Definition y(t-1) = y(t-1)
GAM0(eq,vy1)    = 1;
GAM1(eq,vy)    = 1;
eq=eq+1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%EXP ERROR
GAM0(eq,vpi)    = 1;
GAM1(eq,vEpi)   = 1;
PPI(eq,1)       = 1;

eq=eq+1;
%%EXP ERROR
GAM0(eq,vy)     = 1;
GAM1(eq,vEy)    = 1;
PPI(eq,2)       = 1;

eq=eq+1;
%%EXP ERROR
GAM0(eq,vb)     = 1;
GAM1(eq,vEb)    = 1;
PPI(eq,3)       = 1;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eq=eq+1;
% Auxiliary Process with sunspot on inflation  vw1 = (1/alph1)*vw1(-1) - esuns1 + (eta_1)
GAM0(eq,vw1)    = 1;
GAM1(eq,vw1)    = (1/alph1);
PSI(eq,esuns1)  =  1; 
PPI(eq,1)       = -1;

eq=eq+1;
% Auxiliary Process with sunspot on output  vw2 = (1/alph2)*vw2(-1) - esuns2 + (eta_2)
GAM0(eq,vw2)    = 1;
GAM1(eq,vw2)    = (1/alph2);
PSI(eq,esuns2)  =  1; 
PPI(eq,2)       = -1;

% 






    
    
    