%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================

% number of observation variables
ny = 3;
if msel(15)==1 && RC(1)==1
    % observation eq indices 
    eq_y   = 1;
    eq_pi  = 2;
    eq_r   = 3;

    % variable indices
    vy  = 1;
    vpi = 2;
    vR  = 3;
    vq  = 4;
    vb  = 5;
    vuq = 6;
    vus = 7;
    vy1 = 8;

   


    % names to parameters
    pdt  = p_ss(1,1);
    fub  = p_ss(2,1);
    rr_st = fub*pdt;
    pi_st = p_ss(3,1);
    
    DD(eq_y,1)   = pdt;
    DD(eq_pi,1)  = pi_st;
    DD(eq_r,1)   = rr_st + pi_st;



    ZZ = zeros(ny,13);
    ZZ(eq_y,vy)      =  1; ZZ(eq_y,vy1)      =  -1;
    ZZ(eq_pi,vpi)    =  1;
    ZZ(eq_r,vR)      =  1;
end

if RC(2)==1
    nep = 5;
else
    nep = 4;
end

% shock indices
e_R  = 1;
e_q  = 2;
e_s  = 3;
e_suns1 = 4;
e_suns2 = 5;

% Covariance matrices
% Structural shocks
QQ = zeros(nep,nep,nsc);
for ic=1:nsc
    sigR       = p_er(1,ic);
    sigq       = p_er(2,ic);
    sigs       = p_er(3,ic);
    sigsuns1   = p_er(4,ic);
    sigsuns2   = p_er(5,ic);
    
    r_suns1_R  = p_er(6,ic);
    r_suns1_q  = p_er(7,ic);
    r_suns1_s  = p_er(8,ic);
    r_suns2_R  = p_er(9,ic);
    r_suns2_q  = p_er(10,ic);
    r_suns2_s  = p_er(11,ic);
    
    r_suns2_1  = o_er(1,ic);
    

    QQ(e_R,e_R,ic) = sigR^2;
    QQ(e_q,e_q,ic) = sigq^2;
    QQ(e_s,e_s,ic) = sigs^2;
    QQ(e_suns1,e_suns1,ic) = sigsuns1^2;
    QQ(e_suns2,e_suns2,ic) = sigsuns2^2;
    
    QQ(e_suns1,e_R,ic) = r_suns1_R*sigsuns1*sigR;
    QQ(e_R,e_suns1,ic) = QQ(e_suns1,e_R,ic);
    QQ(e_suns1,e_q,ic) = r_suns1_q*sigsuns1*sigq;
    QQ(e_q,e_suns1,ic) = QQ(e_suns1,e_q,ic);
    QQ(e_suns1,e_s,ic) = r_suns1_s*sigsuns1*sigs;
    QQ(e_s,e_suns1,ic) = QQ(e_suns1,e_s,ic);
    
    QQ(e_suns2,e_R,ic) = r_suns2_R*sigsuns2*sigR;
    QQ(e_R,e_suns2,ic) = QQ(e_suns2,e_R,ic);
    QQ(e_suns2,e_q,ic) = r_suns2_q*sigsuns2*sigq;
    QQ(e_q,e_suns2,ic) = QQ(e_suns2,e_q,ic);
    QQ(e_suns2,e_s,ic) = r_suns2_s*sigsuns2*sigs;
    QQ(e_s,e_suns2,ic) = QQ(e_suns2,e_s,ic);
    
    if msel(22)==1
            %Reduced-form correlation between suspot shocks: compute it in two equivalent ways
            %1) general matrix form
            QQ21 = QQ(4:5,1:3);
            A11 = sqrtm(QQ(1:3,1:3));
            A21 = QQ21/A11';
            for kk =1:size(A21,2)
                QQ(e_suns2,e_suns1,ic) = QQ(e_suns2,e_suns1,ic) + A21(1,kk)*A21(2,kk); % + A21(1,2)*A21(2,2) + A21(1,3)*AA21(2,3))
            end
            QQ(e_suns1,e_suns2,ic) = QQ(e_suns2,e_suns1,ic);
    elseif msel(22)==2
        QQ(e_suns2,e_suns1,ic) = r_suns2_1*sigsuns2*sigsuns1;
        QQ(e_suns1,e_suns2,ic) = QQ(e_suns2,e_suns1,ic);
    end

%

    QQ = (QQ+QQ')/2;
    
    d = eig(QQ);
    issemiposdef = all(d>=0) ;
    if issemiposdef==0
        RC(1) = -3;
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Observation equation error
UU = zeros(ny,ny);  %w/o measurement error


VV = zeros(nep,ny);
nreg=size(TT,3);
CC=[];


