%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
% This code produces:   (1) Parameter estimates
%                       (2) Raftery-Lewis Diagnostics for each parameter chain
%==========================================================================
clear 
clc
close all

defpos = [200, 200, 800, 500];
set(0,'defaultfigurepos',defpos);

addpath Code_bn
addpath Bianchi_matlab
addpath Bianchi_matlab_others/gibbs
addpath Bianchi_matlab_others/util
addpath Bianchi_matlab_others/distrib

figure_setting

skip=10     % If different from 1, keep one every "skip" draws 
lbou=5;    % Choose 5 for 90% error bands
           % Choose 16 for 68% error bands
           
acflag=50;
direct=1;

rescaling_ss=[1,2,4];

% Start from i2, use mixture
fname_main='Results_bn/mcmc/mcmc_dir_ind_1002_ss2_i2_rhoR2_all_mix_1010';

st_file=1;
% end_file= ;

load(fname_main,'fname','modename','extr','msel','rr')
end_file=rr;
fname=fname(st_file:end_file,:);
NF=size(fname,1);
clear ii

VPARAa = [];POMa = [];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for ii=1:NF
    load(fname(ii,:),'VPARA','POM')

    VPARAa=[VPARAa,VPARA];

%     keyboard
    POMa=[POMa;POM];
end
NGjump=size(VPARA,2);

clear VPARA POM
ind=skip:skip:NGjump*NF;
ind2=skip:skip:NGjump;

VPARAa=VPARAa(:,ind);
POMa=POMa(ind);

NG2=size(VPARAa,2);

load(modename(1,:),'abc')
load(modename(1,:),'pnames_all','msel','PRSUM','SUBV','TRSPEC','vnames2')

ubou=100-lbou;
nsb=msel(10);nsc=msel(11);

% Summary statistics
VPARAs(:,1)=abc;
VPARAs(:,2)=mean(VPARAa,2);
VPARAs(:,3:4)=squeeze(prctile(VPARAa,[lbou,ubou],2));

eval(pnames_all)
pnames_coda=pnames_sp(:,[repmat(SUBV{1},nsb,1);SUBV{2}]);
pnames_coda=([pnames_coda,pnames_ss(:,SUBV{5})]);
pnames_coda=([pnames_coda,pnames_se(:,repmat(SUBV{7},nsc,1))]);
pnames_coda=([pnames_coda,pnames_se(:,repmat(SUBV{8},nsc,1))]);
if size(SUBV{11},1)>0
    pnames_coda=([pnames_coda,pnames_oe(:,SUBV{11})]);
end

prinfo = (PRSUM{10});
if lbou==5
    cnames=['Mode';'Mean';'  5 ';' 95 ';'Type';'Mean';'Std '];
elseif lbou==16
    cnames=['Mode';'Mean';' 16 ';' 84 ';'Type';'Mean';'Std '];    
elseif lbou==99
    cnames=['Mode';'Mean';' 16 ';' 84 ';'  5 ';' 95 ';'Type';'Mean';'Std '];    
end

disp('%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('Parameter estimates')
r=printresults3([VPARAs,prinfo],cnames,pnames_coda);disp(r)

disp('--------------------------------------')
disp('--------------------------------------')
disp('Type return for convergence diagnostic')
disp('--------------------------------------')
disp('--------------------------------------')

% info.q = Raftery quantile (default = 0.025)
info.q = .05;
% info.r = Raftery level of precision (default = 0.005)
info.r = .02;
% info.s = Rafterty probability for r (default = 0.950)
info.s = .90;
% info.p1 = 1st % of sample for Geweke chi-sqr test (default = 0.2)
info.p1 = .45;
info.p2 = .45;

pnames_coda = convertStringsToChars(pnames_coda);
pnames_coda2(size(abc,1),:)='a';
coda(VPARAa',pnames_coda2,info)

nd =  size(VPARAa,2);nirf = 20;ny=4;nsh=5;nS=13;
ssirfall_y_all = nan(nirf,ny,nsh,nd);
ssirfall_s_all = nan(nirf,nS,nsh,nd);
vd_all = nan(ny,nsh+2,nd);
for id = 1:nd
    msel(31) = 3;
	[lh,sp,li,pr,MAT,prdet,lidet,flag]=blocks_ftpl7_all2([],[],VPARAa(:,id),[],extr,PRSUM,msel,TRSPEC,SUBV,1);
    [ TT,QQ,RR,UU,DD,ZZ,~,~,snames,shnames ] = from_MAT_to_T(MAT);
    DD(4) = DD(3)-DD(2);
    ZZ(4,3) = 1;ZZ(4,12) = -1;
    UU(4,4)=0;
    MAT{4} = UU;MAT{5} = DD;MAT{6} = ZZ;
    [ssirfall_y,ssirfall_s] = dsge_imprf3( TT,QQ,RR,ZZ,nirf );
    ssirfall_y_all(:,:,:,id) = ssirfall_y;
    ssirfall_y_all(:,1,:,id) = cumsum(ssirfall_y(:,1,:),1);
    ssirfall_s_all(:,:,:,id) = ssirfall_s;
    
    [vd,vol]=vardec5_from_MAT8_chol(MAT,1,0);
    vd_all(:,:,id) = vd(:,:,1,1);

end
vnames2{4}='Real FFR';
ssirfall_s_all=round_plus(ssirfall_s_all,8);
ssirfall_s_p = prctile(ssirfall_s_all,[50,5,95,16,84],4);
ssirfall_y_p = prctile(ssirfall_y_all,[50,5,95,16,84],4);
vd_p = prctile(vd_all,[50,5,95],3);
% ssirfall_y_p = permute(ssirfall_y_p,[1,3,2,4]);

shnames{1}='$\varepsilon_{i}$';
shnames{2}='$\varepsilon_{q}$';
shnames{3}='$\varepsilon_{s}$';
shnames{4}='$\nu_{\pi}$';
shnames{5}='$\nu_{y}$';

vnames3{1}='GDP';vnames3{2}='Inflation';
vnames3{3}='FFR';vnames3{4}='Real FFR';

[ tt ] = dsge_imprf_plot_bands( ssirfall_s_p,snames,shnames );
[ tt ] = dsge_imprf_plot_bands( ssirfall_y_p,vnames2,shnames );
order = [1,4,5,2,3];

temp = ssirfall_y_p(:,:,:,order);
[ tt ] = dsge_imprf_plot_shades2( temp,vnames3,shnames );


for ii = 1:5
    ssirfall_y_p_adj(:,:,1:2,ii) = ssirfall_y_p(:,:,2:3,ii); 
    ssirfall_y_p_adj(:,:,3,ii) = ssirfall_y_p(:,:,1,ii);
    ssirfall_y_p_adj(:,:,4:5,ii) = ssirfall_y_p(:,:,4:5,ii); 
end
shnames_adj{1}='$\varepsilon_{q}$';
shnames_adj{2}='$\varepsilon_{s}$';
shnames_adj{3}='$\varepsilon_{i}$';
shnames_adj{4}='$\nu_{\pi}$';
shnames_adj{5}='$\nu_{y}$';

[ tt ] = dsge_imprf_plot_bands( ssirfall_y_p_adj,vnames2,shnames_adj );

order = [1,4,5,2,3];

temp = ssirfall_y_p_adj(:,:,:,order);
[ tt ] = dsge_imprf_plot_shades2( temp,vnames3,shnames_adj );

