%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
% This code produces Figure 1 and Table 3.
%==========================================================================

close all
clc
clear

addpath Code_bn
addpath Bianchi_matlab
addpath Bianchi_matlab_others/gibbs
addpath Bianchi_matlab_others/util
addpath Bianchi_matlab_others/distrib

figure_setting

fname_files{1}='Initial/first/mcmc_dir_ind_1002_ss2_i2_rhoR2_all_mix_s0_first_1001';
fname_files{2}='Initial/first/mcmc_dir_ind_1002_ss2_i2_rhoR2_all_rw_s0_first_1001';
fname_files{3}='Initial/first/mcmc_dir_ind_1002_ss2_i2_rhoR2_all_mix_s1_first_1001';
fname_files{4}='Initial/first/mcmc_dir_ind_1002_ss2_i2_rhoR2_all_rw_s1_first_1001';

for ij=1:4
    load(fname_files{ij},'FIRST')
    FIRSTALL(:,ij)=FIRST;
    
end

f=figure('name','Number of draws before moving to i2','numbertitle','off');

subplot(1,2,1);histogram(FIRSTALL(:,1),50,...
    'Normalization','probability');
hold on
histogram(FIRSTALL(:,2),50, 'EdgeColor', 'green', 'FaceColor',  'green', 'FaceAlpha', 0.2,...
    'Normalization','probability');
title('Starting from determinacy')
ylim([0 .8])
ylabel('Probability');xlabel('Number of draws')
legend('Mixture','Random walk')
subplot(1,2,2);histogram(FIRSTALL(:,3),50,...
    'Normalization','probability');
hold on
histogram(FIRSTALL(:,4),50, 'EdgeColor', 'green', 'FaceColor',  'green', 'FaceAlpha', 0.2,...
    'Normalization','probability');
title('Starting from one degree of indeterminacy')
ylim([0 .8]);xlabel('Number of draws')
legend('Mixture','Random walk')

FIRSTST(1,:) = nanmean(FIRSTALL,1);
FIRSTST(2:4,:) = prctile(FIRSTALL,[50,5,95]);
FIRSTST(5,:) = sum(FIRSTALL>=10^4)/size(FIRSTALL,1);
FIRSTST(6,:) = sum(FIRSTALL>=10^5)/size(FIRSTALL,1);

FIRSTST=FIRSTST';

cases{1} = 'MIX S0';cases{2} = 'RW S0';cases{3} = 'MIX S1';cases{4} = 'RW S1';
cnames{1} = 'Mean';cnames{2} = 'Median';cnames{3} = '5%';cnames{4} = '95%';
cnames{5} = 'P(nd>10^4)';cnames{6} = 'P(nd>10^5)';

disp(' ')
disp('================================================')
disp('This table reports summary statistics for the distribution')
disp('of the number of draws necessary to jump to the i2 region')
disp('using the RW and MIX algorithms and for different starting values')
r=printresults3(FIRSTST,cnames,cases);disp(r)
