%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
% This code computes the number of draws necessary to cross the 
% threshold for two degrees of indeterminacy 
% for the first time using the hybrid algorithm ("Mixture")
% described in Section 4 starting from the area of the parameter
% space characterized by one degree of indeterminacy
%==========================================================================

clc
clear
close all

addpath Code_bn
addpath Bianchi_matlab
addpath Bianchi_matlab_others
addpath Bianchi_matlab_others/gibbs
addpath Bianchi_matlab_others/util
addpath Bianchi_matlab_others/distrib

warning off

% Name of files with peaks of posterior
modename(1,:)='Initial/mode_1002_ss2_i2_rhoR2_2045';
modename(2,:)='Initial/mode_1002_ss2_i1_rhoR2_2007';
modename(3,:)='Initial/mode_1002_ss2_i0_rhoR2_2001';

ipeak0 = 2; % Starting peak. If empty, random choice
shape = 2;  % 0: Normal, 1: t-distribution, 2: Mixture of normals
probmix = 0.1; % Probability of mixture vs random walk
suffix_mcmc = 'all_mix_s1_first_';



numberf = 1000;%Number to identify MCMC
NG = 100000;
NF = 1;
wuit = 0;     
jump = 1;

NCH=2000;
drawalpha=1


%------------------------------------------------------------
% Vector for normalization sigmas
vectc=2;
% Vector for normalization structural parameters
vectb=1;


%------------------------------------------------------------
% Use blocks: Structural parameters Vs stochastic volatilities/OE/HC
npeaks = size(modename,1);
% Scaling factor for draws from local peaks
ccs = [(10^-3)*1,(10^-3)*1,(10^-3)*1];
% Scaling factor for RW
ccbasic = .3*(10^-3);
% Probabilities of drawing from different proposal distributions
prpeaks = ones(npeaks,1)/npeaks;

if shape ==1
    % Degrees of freedom for different proposal distributions
    df = ones(1,npeaks)*3;
elseif shape == 2
    % Prob to draw from normal with large variance
    df = [.1,.1]; 
    df(2,1:npeaks) = 5;
elseif shape == 0
    df = nan(2,npeaks);
end



rescale=[]

% File names
load(modename(1,:),'subfx','foldername')
for jj=1:NF
    momo = 'mcmc_dir_ind_';
    nunu = num2str(jj+numberf) ;
    fname(jj,:)= strcat(foldername,'mcmc/', momo, subfx, suffix_mcmc, nunu);
end
tempmcmc = strcat(foldername,'mcmc/','zz',momo, subfx, suffix_mcmc, nunu);
fname_all = strcat(foldername,'mcmc/', momo, subfx, suffix_mcmc, nunu);

load(modename(1,:),'fcnall','dates','ss','msel','PRSUM','SUBV','extr')
load(modename(1,:),'nsb','nsc','smb','smc','TRSPEC')

msel(2)=-1;
for ip=1:npeaks
    load(modename(ip,:),'abc','yt','pomo','SUBV')
    load(modename(ip,:),'HHH')
    SUBV_all{ip} = SUBV;
 
    [lh,~,~,~,~,~,~,flag]=blocks_ftpl7_all2([],yt,abc,[],extr,PRSUM,msel,TRSPEC,SUBV,1);
    pomode = -lh;
    if abs(pomo-pomode)>.1; 'Error in function used to calculate posterior'; ddddd; end

    meanprop(:,ip) = abc;

    iHprop(:,:,ip) = HHH;
    pomoall(ip)=pomo;
end


pause(2)
K = size(abc,1);
clear abc HHH iH

iHbasic = mean(iHprop(:,:,1),3);
iHprop = repmat(iHbasic,1,1,3);



%%%%%%%%%%%%%%%%%%
% c) Bounds
BOU=trspec_to_bou2(TRSPEC,SUBV,msel);
BOUv=TRSPEC{10}(:,2:3);



ipeak_now = ipeak0;
clear ip

ratio=zeros(NG/jump,2,NCH);
IPEAK_NOW=zeros(NG/jump,NCH);

FIRST=nan(NCH,1);
for ic=1:NCH
    
    if mod(ic,20)==0;disp('Chain');disp(ic/NCH);disp(median(FIRST(1:ic-1)));end
    

    %-------------------------------------------------------
    % Make initial draw
    % Initialize chain
    ipeak_now = ipeak0;
    poo=-10^10;
    pomo = pomoall(ipeak0);
    meanproptemp = meanprop(:,ipeak0);
    iHproptemp = iHprop(:,:,ipeak0);
    SUBV = SUBV_all{ipeak0};
    while poo<=pomo-200

        vparaT=meanproptemp+real(2*ccbasic*iHproptemp*randn(K,1));
        poo=-blocks_ftpl7_all2([],yt,vparaT,[],extr,PRSUM,msel,TRSPEC,SUBV,1);
    end



    poo0=poo;
    vpara0=vparaT;
    vpara=vparaT;
    %-------------------------------------------------------
    
    % MCMC algorithm
    r=1;
    rr=1;
    RJ = nan(NG/jump,3);
    VPARA = nan(K,NG/jump);
    POM=zeros(NG/jump,1);

    while r<NG+wuit+1 && rr<NF+1 && ipeak_now~=1

        bini_rjrule_mixture


        if r>wuit && mod(r,jump)==0
            

            % Save results
            RJ((r-wuit)/jump,:)=rj;
            POM((r-wuit)/jump,:)=poo;
            IPEAK_NOW((r-wuit)/jump,:)=ipeak_now;

            VPARA(:,(r-wuit)/jump)=vpara;
            if mod(r,5000)==0
                disp('------');disp([r/NG,rr,ic]);
                disp('RJ: ');disp(sum(RJ(1:(r-wuit)/jump,1))/((r-wuit)/jump));
            end

            if r==NG 
                save(fname(rr,:))
                if rr<NF
                    r=0;


                end
                rr=rr+1;
            end
        end


        r=r+1;
        if mod(r-wuit-1,1000)==0;save(tempmcmc);end
    end
    FIRST(ic)=r-1;
end
save(fname_all)

