%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
% This code computes the posterior mode conditional on the region of 
% one-degree indeterminacy for Gali's model.
%==========================================================================

clc
clear
close all
addpath Code_bn
addpath Bianchi_matlab

warning off

NF = 50;         % Number of iterations
subfx = '1002_ss2_i2_rhoR2_';   % Suffix to save results
foldername = 'Results_bn/';





msel(1)=1;  %1-> Post 82
            %2-> Pre 82
            %3-> Whole sample
msel(2)=1;  %Not used
msel(3)=2;  %Not used
msel(4)=0;  %Not used
            
msel(5)=2;  % msel(5)=1 -> Indet/determinacy
            % msel(5)=2 -> Only determinacy
            % msel(5)=2 -> Indeterminacy with sunspost shock
msel(6)=1002;
            %1001-> Model of LS2004
            %1002-> Gali model with asset bubbles
            
msel(7)=2; %Not used
msel(8)=2; %Not used
           
           
msel(9)=2; %Not used
           
msel(10)=1; %Not used
msel(11)=1; %Not used

msel(12)=0; %Not used
            
msel(13)=3; %Not used
            
msel(14)=1; %Not used
            
msel(15)=1; %Not used
            
msel(16)=1; %Not used
            
msel(17)=2; %Not used
            
msel(18)=1; %Not used
            
msel(19)=1; %Not used
            
msel(20)=1; %Not used

msel(21)=1; %Not used
            
msel(22)=2; % Model specific options
            
            % BINI: 1->restricted correlation sunspot
            % BINI: 2->No restricted correlation sunspot
            
            
msel(23)=1; %Not used

msel(24)=2; %Not used
            
msel(25)=0; %Not used
            
msel(26)=0; %Not used


            
msel(31)=0; % 0 -> Posterior
            % 1 -> Prior
            % 2 -> Likelihood
            % 3 -> Inside boundaries and existence of solution
            
msel(32)=11;%Not used

msel(33)=20;%Not used
            
msel(34)=0; %Not used
            
msel(35)=1; %Not used


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBV:
% 1-14 : Subvecors for ms,fc,fx parameters
% 15   : Not used
% 16   : Matrices for law of motion
% 18-19: Normalization
nsb=msel(10);
nsc=msel(11);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


POMO=-inf*ones(NF,1);
for jj=1:NF
    nunu = num2str(jj+2000) ;
    fn(jj,:)= strcat(foldername, 'mode_',subfx, nunu);
end
tempf=strcat(foldername,'zzmode_temp_', subfx);
fnsummary = strcat(foldername, 'mode_',subfx, 'summary');
SUBV{20}=strcat('zzfname_', subfx);




              



nunu = num2str(msel(6)) ;
% Data
varspec_all=strcat('varspec_',nunu,'_ss2');
% Subvectors m files
subv_sel_all=strcat('subv_sel_',nunu,'_i2_rhoR2');
% Priors m files
priors_sp_all=strcat('priors_sp_',nunu,'');
priors_ss_all=strcat('priors_ss_',nunu,'');
priors_se_all=strcat('priors_se_',nunu,'');
priors_oe_all=strcat('priors_oe_',nunu,'');
if nsc>1;priors_hc_all=strcat('priors_hc_',nunu);end
if nsb>1;priors_hb_all=strcat('priors_hb_',nunu);end
if msel(32)==1
    priors_zr_all=strcat('priors_zr_',nunu);
end
 % Transformation m files
prtr_all=strcat('prtr_',nunu,'');
% Starting values
pmsv_oe_all=strcat('pmsv_draw');
pmsv_sp_all=strcat('pmsv_draw');
pmsv_ss_all=strcat('pmsv_draw');
pmsv_se_all=strcat('pmsv_draw');
% Parameter, var and sh names
pnames_all=strcat('pnames_',nunu);
vnames_all=strcat('vnames_',nunu);
% [1] Data
eval(varspec_all)

% [2] Subvectors
eval(subv_sel_all)

% [3] Priors
PRSUM{1}=feval(priors_sp_all,msel);
PRSUM{2}=feval(priors_ss_all,yt);
PRSUM{3}=feval(priors_se_all,msel);
PRSUM{4}=feval(priors_oe_all,msel);
if nsc>1;PRSUM{5}=feval(priors_hc_all,msel);end
if nsb>1;PRSUM{6}=feval(priors_hb_all,msel);end
if nsb>1;PRSUM{7}=feval(priors_hb_all,msel);end
if msel(32)==1
    PRSUM{8}=feval(priors_zr_all,yt);
end
PRSUM{10}= from_prsum_to_prvec( PRSUM,SUBV );

% [4] Transformations and bounds
eval(prtr_all);
TRSPEC{10}= from_trspec_to_trvec2( TRSPEC,SUBV,nsb,nsc,msel );

% [5] Starting values
clear parai
% [5.1] Structural parameters
for i=1:nsb
    parai{1}(:,i) = feval(pmsv_sp_all,PRSUM{1}(:,:,i));
end
% [5.2] Steady states
parai{2}(:,1) = feval(pmsv_ss_all,PRSUM{2});
% [5.3] Structural errors
for i=1:nsc
    parai{3}(:,i) = feval(pmsv_se_all,PRSUM{3}(:,:,i));
end
% [5.4] Observation errors
parai{4} = feval(pmsv_se_all,PRSUM{4});
% [6] Parameter names
eval(pnames_all)
eval(vnames_all)


fcn='blocks_ftpl7';
fcnall='blocks_ftpl7_all2';

jhk=1;
while jhk<=NF
    % Optimization parameters
    impr0=10.^-6;
    ft1=10.^100;
    impr=10.^100;
    crit=.1;
    nit=10;
    grad=[];
    maxoe=2;
    nit2 = nit;
    crit2 = crit;
    maxnit = 20;
    wrandIn=0;

    f0=10.^100;
    while f0>100000 
        % [5] Starting values
        clear parai
        % [5.1] Structural parameters
        for i=1:nsb
            parai{1}(:,i) = feval(pmsv_sp_all,PRSUM{1}(:,:,i));
        end
        % [5.2] Steady states
        parai{2}(:,1) = feval(pmsv_ss_all,PRSUM{2});
        % [5.3] Structural errors
        for i=1:nsc
            parai{3}(:,i) = feval(pmsv_se_all,PRSUM{3}(:,:,i));
        end
        % [5.4] Observation errors
        parai{4} = feval(pmsv_oe_all,PRSUM{4});
        [aaa1,aaa2,bbb,ccc,ddd] = para_to_vpara3( parai,SUBV,msel );

        if nsc>1;eee=rand(nsc*(nsc-1),1);else eee=[];end
        if nsb>1 && msel(25)~=-5;fff=rand(nsb*(nsb-1),1);else fff=[];end
        SUBV{16}{7}=[];SUBV{16}{1}=[];


        [f0,~,l0,~,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
        f0

    end
    extr_define



    % Initial hessians
    abc=[aaa1;aaa2;bbb;ccc;ddd;eee;fff];
    HHH=diag(abc)*(10^-5);
    [Ha1,Ha2,Hb,Hc,Hd,He,Hf]=from_HHH_to_Ht(HHH,extr);
    K=size(abc,1);

    HHH05=real(sqrtm(HHH));
    ii=0;jj=0;
    while ii<100 && jj<20
        zzz=abc+0.5*HHH05*randn(K,1);
        [z1,z2,z3,z4,z5,z6,z7]=from_abc_to_max(zzz,extr);
        [ft,~,lt]=feval(fcn,z1,yt,z2,z3,z4,z5,z6,z7,1,PRSUM,msel,TRSPEC,SUBV);
        if ft<f0
            abc=zzz;
            aaa1=z1;aaa2=z2;
            bbb=z3;ccc=z4;ddd=z5;
            eee=z6;fff=z7;f0=ft
            if sum(msel(13)==[3,33,34,35]) || msel(13)<0
            [~,~,~,~,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
            end
            jj=jj+1;
        end
        ii=ii+1;

    end


    [f0,sp,li,pr,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
    disp('-------------------------------------------')
    disp('Posterior at starting values')
    disp(-f0)
    disp('-------------------------------------------')
    disp('Likelihood at starting values')
    disp(li)
    disp('-------------------------------------------')
    disp('Prior at starting values')
    disp(pr)


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    [f0,~,~,~,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    HHH=blockdiag(Ha1,Ha2,Hb,Hc,Hd,He,Hf);
    abc=[aaa1;aaa2;bbb;ccc;ddd;eee;fff];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    npara=round(size(HHH,1)/4);
    crit3=crit2;
    nit3=6;
    HHH2=zeros(size(HHH));
    xabc=zeros(size(abc));


    while  min(sum(HHH2))<npara && wrandIn<30
        if rand<.5
            ind1=randperm(extr_sp+extr_ot+size(fff,1));
            ind1=sort(ind1(1:npara));
        else
            [ox,ix]=sort(xabc(extr_sp_hb));
            ind1=extr_sp_hb(ix(1:npara));
%                 ind1=ix(1:npara);    
        end
        disp('--------------------------')
        disp('Random from all parameters')
        disp('--------------------------')
        zz1=abc(ind1);H0=HHH(ind1,ind1);
        bl2=1;
        [ft,xt,gt,Ht] = csminwel(fcnall,zz1,H0,grad,crit3,nit3,yt,abc,ind1,extr,PRSUM,msel,TRSPEC,SUBV,bl2);
        if ft<=ft1
           abc(ind1)=xt;ft1=ft;[ Ht ] = checkH( Ht );
           [aaa1,aaa2,bbb,ccc,ddd,eee,fff]=from_abc_to_max(abc,extr);

           HHH(ind1,ind1)=Ht;
           [Ha1,Ha2,Hb,Hc,Hd,He,Hf]=from_HHH_to_Ht(HHH,extr);
           if sum(msel(13)==[3,33,34,35]) || msel(13)<0
            [~,~,~,~,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
           end
           Hind33=ones(size(H0));
           HHH2(ind1,ind1)=HHH2(ind1,ind1)+Hind33;
           xabc(ind1)=xabc(ind1)+1;
        end
        wrandIn=wrandIn+1;
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    coin_bl=1;
    coin_dd=1;
    while impr>impr0 
        ft0=ft1;
         
        wrand=1;
        abc=[aaa1;aaa2;bbb;ccc;ddd;eee;fff];
        xabc=zeros(size(abc));
        while min(xabc)<=2
            disp(tempf)
               abc=[aaa1;aaa2;bbb;ccc;ddd;eee;fff];
               maxpara=min(extr_sp,10);
               npara=randi([5,maxpara],1);
               
               if rand<.9 % 90% of cases max wrt transition matrix 
                   if sum(xabc)==0 || rand<.1
                       
                    ind1=shuffle(extr_sp_hb);
                   else
                    [ox,ix]=sort(xabc(extr_sp_hb));
                    ind1=extr_sp_hb(ix(1:npara));
                    
                   end
                   ind1=sort(ind1(1:npara));
                    disp('--------------------------')
                    disp('Random from SP and SS + HB')
                    disp('--------------------------')
               else % 10% of cases:
                    if rand<.2 % 2% of cases NO max wrt transition matrix
                        ind1=extr_sp_hb;

                        disp('----------------')
                        disp('All SP + SS + HB')
                        disp('----------------')
                    else % 8% of cases random from all parameters

                        ind1=randperm(extr_sp+extr_ot+size(fff,1));
                        ind1=sort(ind1(1:maxpara));%ind1=sort(ind1(1:npara));
                        disp('--------------------------')
                        disp('Random from all parameters')
                        disp('--------------------------')
                    end
               end
               zz1=abc(ind1);H0=HHH(ind1,ind1);
               xabc(ind1)=xabc(ind1)+1;
               bl2=1;
               [ft,xt,gt,Ht] = csminwel(fcnall,zz1,H0,grad,crit2,nit2,yt,abc,ind1,extr,PRSUM,msel,TRSPEC,SUBV,bl2);

               
               if ft<=ft1
                   abc(ind1)=xt;ft1=ft;[ Ht ] = checkH( Ht );
                   [aaa1,aaa2,bbb,ccc,ddd,eee,fff]=from_abc_to_max(abc,extr);

                   HHH(ind1,ind1)=Ht;
                   [Ha1,Ha2,Hb,Hc,Hd,He,Hf]=from_HHH_to_Ht(HHH,extr);
                   if sum(msel(13)==[3,33,34,35]) || msel(13)<0
                    [~,~,~,~,SUBV{16}]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
                   end
               end

           disp('-----------------------------')
           disp('Random selection of SE and HC')
           disp('-----------------------------')
           abc=[aaa1;aaa2;bbb;ccc;ddd;eee;fff];
           maxpara=min(extr_ot,10);
           npara=randi([min(maxpara,6),maxpara],1);
           ind1=shuffle(extr_nostr);

           ind1=sort(ind1(1:npara));
           xabc(ind1)=xabc(ind1)+1;
           zz1=abc(ind1);H0=HHH(ind1,ind1);
           bl2=1;
           [ft,xt,gt,Ht] = csminwel(fcnall,zz1,H0,grad,crit2,nit2,yt,abc,ind1,extr,PRSUM,msel,TRSPEC,SUBV,bl2);
           if ft<=ft1
               abc(ind1)=xt;ft1=ft;[ Ht ] = checkH( Ht );
               [aaa1,aaa2,bbb,ccc,ddd,eee,fff]=from_abc_to_max(abc,extr);

               HHH(ind1,ind1)=Ht;
               [Ha1,Ha2,Hb,Hc,Hd,He,Hf]=from_HHH_to_Ht(HHH,extr);
    
           end
           wrand=wrand+1;
           save(tempf)
        end 
        
        impr=abs(ft1-ft0);
        crit=max(crit/4,impr0/100);
        crit2=max(crit2/4,impr0/100);
        nit = min(nit+1,maxnit);
        nit2 = min(nit2+1,maxnit);
        coin_bl=rand;
        coin_dd=rand;
        save(tempf)
        



        if impr<2*impr0*(10^4) && msel(35)==0 && dummy_Hmax==1
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            msel(35)=1;
            [ eee ] = trans_H_elem( eee );[ fff ] = trans_H_elem( fff );
            He=0.0001*ones(size(eee,1),size(eee,1));
            Hf=0.0001*ones(size(fff,1),size(fff,1));
            HHH(extr{6},:)=HHH(extr{6},:)/100;
            HHH(:,extr{6})=HHH(:,extr{6})/100;
            HHH(extr{7},:)=HHH(extr{7},:)/100;
            HHH(:,extr{7})=HHH(:,extr{7})/100;
            crit=crit/100;crit2=crit2/100;crit3=crit3/100;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end
    end

    pomo=-ft;
    [fmo,spmo,limo,prmo,MATmo,detmo]=feval(fcn,aaa1,yt,aaa2,bbb,ccc,ddd,eee,fff,1,PRSUM,msel,TRSPEC,SUBV);
    bl=1;z1=aaa1;z2=aaa2;z3=bbb;z4=ccc;z5=ddd;z6=eee;z7=fff;H0=Ha1;
    [p_st_mo,p_ss_mo,p_er_mo,o_er_mo,HBmo,HCmo,HBEmo,EXTRAmo]=bini_vpara_to_para6(aaa1,aaa2,bbb,ccc,ddd,eee,fff,SUBV,msel);
 

    
    K=size(abc,1);
    BIC_po=-2*pomo+K*log(ss);
    BIC_li=-2*limo+K*log(ss);
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if msel(31)~=1 && msel(17)~=-3 % LIkelihood or posterior
        if nsb>1;SMB(:,:,jhk)=smb;end
        if nsc>1;SMC(:,:,jhk)=smc;end
    end
    P_ST_MO(:,jhk,:)=p_st_mo;
    P_ER_MO(:,jhk,:)=p_er_mo;
    P_SS_MO(:,jhk)=p_ss_mo;
    O_ER_MO(:,jhk)=o_er_mo;
    HBMO(:,:,jhk)=HBmo;
    HCMO(:,:,jhk)=HCmo;
    HBEMO(:,:,jhk)=HBEmo;
    ABC(:,jhk)=abc;
    
    
    HHHH(:,:,jhk)=HHH;
    % Store results
    if jhk<=3 || jhk==NF
        POMO(jhk)=pomo;    
        save(fn(jhk,:));
    else
        
        [trh1,trh2]=sort(POMO,'descend');
        if pomo>trh1(3)
            POMO(jhk)=pomo;
            save(fn(jhk,:));
        else
            POMO(jhk)=pomo;
        end
    end
    LIMO(jhk)=limo;
    PRMO(jhk)=prmo;
    jhk=jhk+1;
    save(tempf)
    if jhk<NF
        save(fnsummary)
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end



