%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
if rand<probmix
    peaks = 1; % Draws from distributions centered on local peaks
else
    peaks = 0; % Use RW (but adjust probability of proposal)
end

if peaks
    clear ip
    % Step 1: Select proposal distribution
    ipeak=randdisc(prpeaks,1);%ipeak = 3
    meanproptemp = meanprop(:,ipeak);
    iHproptemp = iHprop(:,:,ipeak);
    ctemp = ccs(ipeak);
    dftemp = df(:,ipeak);
    SUBVtemp = SUBV_all{ipeak};

    pon = -10^10;
    while pon<-10^4   % RC=1 imples that a solution exists!
        % Step 2: Make a draw from proposal distribution (With 0 we use normal)
        vparaN=bini_proposec3(meanproptemp,iHproptemp,BOUv,ctemp,shape,dftemp);
        % Step 3: Evalute posterior for proposed parameters
        [npo,~,lin,~,MAT,~,~,flag]=feval(fcnall,[],yt,vparaN,[],extr,PRSUM,msel,TRSPEC,SUBVtemp,1); 
        pon=-npo;

    end

else
    clear ip
    % Step 1: Select proposal distribution
    meanproptemp = vpara;
    iHproptemp = iHbasic;
    ctemp = ccbasic;
    ipeak = ipeak_now;


    pon = -10^10;
    while pon<-10^4   % RC=1 imples that a solution exists!
        SUBVtemp = SUBV;
        % Step 2: Make a draw from proposal distribution (With 0 we use normal)
        vparaN=bini_proposec3(meanproptemp,iHproptemp,BOUv,ctemp);
        % Step 3: Evalute posterior for proposed parameters
        [npo,~,lin,~,MAT,~,~,flag]=feval(fcnall,[],yt,vparaN,[],extr,PRSUM,msel,TRSPEC,SUBV,1); 
        pon=-npo;
        if pon<-10^4 && flag==1
            itry = 0;
            while flag==1 && itry<npeaks && msel(6)==1002
                itry = itry + 1;
                SUBVtemp = SUBV_all{itry};
                [npo,~,lin,~,MAT,~,~,flag]=feval(fcnall,[],yt,vparaN,[],extr,PRSUM,msel,TRSPEC,SUBVtemp,1); 
                pon=-npo;
            end
            if pon<-10^4
                ipeak = itry;
            end
        end
        

    end
end




% Step 4: Compute relative probabilities of the two sets of parameters
% Compute probability of vparaN
% Step 4.1: Basic proposal probability for RW
qbasic = mvnpdf(vparaN,vpara,ccbasic*iHbasic);

% Step 4.2: Proposal probability for mixture of peaks
logq_vparaN = nan(1,npeaks);
logq_vpara = nan(1,npeaks);
q_vparaN = 0;
q_vpara = 0;
for ip = 1:npeaks
    meanproptemp = meanprop(:,ip);
    iHproptemp = iHprop(:,:,ip);
    ctemp = ccs(ip);
    
    if shape==0 % Normal
        logq_vparaN(ip) = mvnpdflog(vparaN,meanproptemp,ctemp*iHproptemp);
        logq_vpara(ip) = mvnpdflog(vpara,meanproptemp,ctemp*iHproptemp);
    elseif shape==1 % t distribution
       
    elseif shape==2 % Mixture of normals
        prop = df(1,ip);rescale = df(2,ip);
        logq_vparaN(ip) = log(prop*mvnpdf(vparaN,meanproptemp,rescale*ctemp*iHproptemp)+...
            (1-prop)*mvnpdf(vparaN,meanproptemp,ctemp*iHproptemp));
        logq_vpara(ip) = log(prop*mvnpdf(vpara,meanproptemp,rescale*ctemp*iHproptemp)+...
            (1-prop)*mvnpdf(vpara,meanproptemp,ctemp*iHproptemp));
    end
end

q_vparaN_norm = exp(logq_vparaN);
q_vpara_norm = exp(logq_vpara);
if sum(q_vparaN_norm)==0 && sum(q_vpara_norm)==0
    if peaks == 1
        baseN = -max(logq_vparaN);
        base = -max(logq_vpara);
        q_vparaN_norm = exp(logq_vparaN+baseN);
        q_vpara_norm = exp(logq_vpara+base);
        q_old_min_q_new=log(q_vpara_norm*prpeaks*probmix+(1-probmix)*qbasic*exp(base))-...
            log(q_vparaN_norm*prpeaks*probmix+(1-probmix)*qbasic*exp(baseN))-base+baseN;
        if sum(q_vparaN_norm)==0 && sum(q_vpara_norm)==0
            ffffffffffffffff
        end
        dddddddddddddd
    else
        q_old_min_q_new = 0;
    end
else
    q_old_min_q_new=log(q_vpara_norm*prpeaks*probmix+(1-probmix)*qbasic)-...
        log(q_vparaN_norm*prpeaks*probmix+(1-probmix)*qbasic);
    
end




% Step 5: Compute treshold
tr = min(1 , exp(pon - poo + q_old_min_q_new))   ; 
cut=rand;


PO=[pon,poo];

if cut < tr
    %   Accept proposed jump
    vpara = vparaN;

    poo = pon;
    lio = lin;
    SUBV=SUBVtemp;
    rj = [0,ipeak,peaks];
    ipeak_now = ipeak;
    if peaks       
        
        q_vpara = q_vparaN;
        
        
         
    end
else
    rj = [1,ipeak,peaks]; 
end



