%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================
function [ rloglh,flag,Atall,Ptall,Phatall,P00,ldet ] = eval_like_fc5( yt,DD,ZZ,TT,RR,QQ,HH, msel )

nstate = size(TT,1);
loglhzero = -1E12;



% Initial mean and variance for the state vector 
At = zeros(nstate,1);
XX=RR*QQ*RR';XX=(XX+XX')/2;
    
    
Pt = real(dlyap(TT,XX)); 
if any(diag(Pt)<0)
    check=0;
else
    check=1;
end
P00=Pt;


ldet = [];
if check==1
    % Filter loop
    t = 1;
    ny=size(yt,2);
    ss=size(yt,1);
    ldet = nan(ss,1);
    if nargout>2
        Atall=zeros(nstate,ss);
        Ptall=zeros(nstate,nstate,ss);
        Phatall=zeros(nstate,nstate,ss);
    end
    In=eye(ny);
    loglh=0;
    riccati_tol=10^-8;
    notsteady=1;
    constant = 0.5*ny*log(2*pi);
    while t <= ss && notsteady
        At1 = At;
        Pt1 = Pt;

        % Forecasting
        alphahat = TT*At1;
        Phat     = TT*Pt1*TT' + XX;
        Phat     = .5*(Phat+Phat');
        % Prediction error
        yhat     = ZZ*alphahat + DD;
        nut      = yt(t,:) - yhat';
        % Prediction error covariance matrix
        Ft       = ZZ*Phat*ZZ' + HH;% 
        Ft       = .5*(Ft+Ft');

        % Log likelihood
        ldet(t) = - constant - 0.5*log(det(Ft)) - 0.5*(nut/Ft)*nut';
        loglh = loglh + ldet(t);
        lx(t)=0.5*nut/Ft*nut';
        % Updating
        Gain = Phat*ZZ'/Ft;
        At = alphahat + Gain*nut';
        Pt = Phat - Gain*ZZ*Phat';
        
        % Store results
        if nargout>2
            Atall(:,t)=At;
            Ptall(:,:,t)=Pt;
            Phatall(:,:,t)=Phat;
        end


        if t>1
            % Check if Gain matrix has reached Steady State
            notsteady = max(abs(Gain(:)-oldGain)) > riccati_tol;
            oldGain=Gain(:);
        else
            oldGain=Gain(:);
        end

        if any(diag(Pt)<-0.0001)
            disp(t)
            t=ss;
            loglh=loglhzero;
            
            disp('Problem with cov matrix in KF eval_like_fc5')
        end

        t = t+1;
    end

    logdetFt=log(det(Ft));
    while t <= ss
        At1 = At;

        % Forecasting
        alphahat = TT*At1;

        % Prediction error
        yhat     = ZZ*alphahat + DD;
        nut      = yt(t,:) - yhat';

        % Log likelihood
        ldet(t) = - constant - 0.5*logdetFt - 0.5*nut/Ft*nut';
        loglh = loglh + ldet(t);
        
        
        % Updating
        At = alphahat + Gain*nut';
    
        % Store results
        if nargout>2
            Atall(:,t)=At;
            Ptall(:,:,t)=Pt;
            Phatall(:,:,t)=Phat;
        end


        if any(diag(Pt)<-0.0001)
            
            t=ss;
            loglh=loglhzero;
%             disp(t)
            disp('Problem with cov matrix in KF eval_like_fc5')
            
        end

        t = t+1;
    end
    
    ck=isnan(real(loglh));
    if ck >= 1
       loglh = loglhzero;
       flag=-2;
    else
       flag=0;
    end

    rloglh=real(loglh);  
else
    rloglh=-1E12*6;
    flag=-3;
    if nargout>2
        Atall=[];
        Ptall=[];
        Phatall=[];
    end
end
%  keyboard

end

