%==========================================================================
% Replication codes for Bianchi-Nicolo' 
% "A Generalized Approach to Indeterminacy in Linear Rational Expectations Models"
% Quantitative Economics
%==========================================================================

function [lnprior,lnprior_det] = priodens2(para, prsum)
% This code computes a prior density for
% the structural parameters of the DSGE models
% pshape: 0 is point mass, both para and pstdd are ignored
%         1 is BETA(mean,stdd)
%         2 is GAMMA(mean,stdd)
%         3 is NORMAL(mean,stdd)
%         4 is INVGAMMA(s^2,nu)
%         5 is Dirichlet (MISSING HERE)
%         6 is Uniform
if size(prsum,1)~=size(para,1)
    disp('Error in priodens2: Sizes are not consistent')
    dddddddddddd
end
pmean=prsum(:,1);
pstdd=prsum(:,2);
pshape=prsum(:,3);

a = 0;
b = 0;

nprio = size(pshape,1);
prioinfo = [zeros(nprio,2),pshape];
lnprior_det = zeros(nprio,1);
% 
i = 1;
while i <= nprio
    if prioinfo(i,3) == 1 % BETA Prior
        a = (1-pmean(i))*pmean(i)^2/pstdd(i)^2 - pmean(i);
        b = a*(1/pmean(i) - 1);
        if para(i)>0 && para(i)<1
        lnprior_det(i) = lpdfbeta(para(i),a,b);        
        else
        lnprior_det(i) = -inf;
        disp('Parameter outside boundaries in priodens2')
        end
    elseif prioinfo(i,3) == 2 % GAMMA PRIOR
        b = pstdd(i)^2/pmean(i);
        a = pmean(i)/b;
        
        if para(i)>0 
            lnprior_det(i) = lpdfgam(para(i),a,b);        
        else
        lnprior_det(i) = -inf;
            disp('Parameter outside boundaries in priodens2')
        end
    elseif prioinfo(i,3) == 3 % GAUSSIAN PRIOR
        a = pmean(i);
        b = pstdd(i);
        lnprior_det(i) = lpdfnor(para(i),a,b);
    elseif prioinfo(i,3) == 4 % INVGAMMA PRIOR
        a = pmean(i); % For inverse gamma these are already transformed
        b = pstdd(i);
        lnprior_det(i) = lpdfig(para(i),a,b);
    elseif prioinfo(i,3) == 6 %Uniform prior
        a = pmean(i); % min
        b = pstdd(i); % max
        lnprior_det(i) = log(1/(b-a)); %abs(1/(b-a));
    end
    prioinfo(i,1) = a;
    prioinfo(i,2) = b;
    i = i+1;
end
lnprior = sum(lnprior_det);







